package Data::Walk::Print;

use Moose::Role;
requires 
	'_get_had_secondary', 
	'_process_the_data';
use MooseX::Types::Moose qw(
        HashRef
        ArrayRef
        Bool
        Str
        Ref
    );######<-------------------------------------------------------  ADD New types here
use version; our $VERSION = qv('0.009_003');
BEGIN{
	if( $ENV{ Smart_Comments } ){
		use Smart::Comments -ENV;
		### Smart-Comments turned on for Data-Walk-Print
	}
}

###############  Package Variables  #####################################################

$| = 1;
my $print_keys = {
    primary_ref     => 'print_ref',
    secondary_ref   => 'match_ref',
};
my $wait;

###############  Dispatch Tables  #######################################################

my	$item_dispatch ={######<-----------------------------------------  ADD New types here
		SCALAR 	=> sub{ return ''; },
		DEFAULT	=> sub{ return $_[1]; },
	};

my	$inter_ref_dispatch ={######<------------------------------------  ADD New types here
		HASH 	=> sub{ return ' => '; },
		DEFAULT	=> sub{ return ''; },
	};

my	$initial_match_statement_dispatch ={######<----------------------  ADD New types here
		HASH => sub{ 
			return (	
				$_[1] ? 
					'Secondary Key Match - ' :
                    'Secondary Key Mismatch - ' 
			); },
		ARRAY => sub{ 
			return (	
				$_[1] ? 
					'Secondary Position Exists - ' :
                    'Secondary Position Does NOT Exist - ' 
			); },
		DEFAULT	=> sub{ return ''; },
	};


###############  Public Attributes  #####################################################

has 'match_highlighting' =>(
    is      	=> 'ro',
    isa     	=> Bool,
    writer  	=> 'set_match_highlighting',
	predicate	=> 'has_match_highlighting',
	reader		=> 'get_match_highlighting',
	clearer		=> 'clear_match_highlighting',
    default 	=> 1,
);

###############  Public Methods  ########################################################

sub print_data{
    ### <where> - Made it to print
    ##### <where> - Passed input  : @_
    my  $self = $_[0];
    my  $passed_ref = 
            ( @_ == 2 and 
                (   ( is_HashRef( $_[1] ) and !( exists $_[1]->{print_ref} ) ) or
                    !is_HashRef( $_[1] )                                            ) ) ? 
                { print_ref => $_[1] }  :
            ( @_ == 2 and is_HashRef( $_[1] ) ) ? 
                $_[1] : 
                { @_[1 .. $#_] } ;
    ##### <where> - Passed hashref: $passed_ref
    @$passed_ref{ 'before_method', 'after_method' } = 
        ( '_print_before_method', '_print_after_method' );
    ##### <where> - Start recursive passing with  : $passed_ref
    $passed_ref = $self->_process_the_data( $passed_ref, $print_keys );
    ### <where> - End recursive passing with    : $passed_ref
    return 1;
}

###############  Private Attributes  ####################################################

has '_pending_string' =>(
    is          => 'ro',
    isa         => Str,
    writer      => '_set_pending_string',
    clearer     => '_clear_pending_string',
    predicate   => '_has_pending_string',
	reader		=> '_get_pending_string',
);

has '_match_string' =>(
    is          => 'ro',
    isa         => Str,
    writer      => '_set_match_string',
    clearer     => '_clear_match_string',
    predicate   => '_has_match_string',
);

###############  Private Methods / Modifiers  ###########################################

sub _print_before_method{
    my ( $self, $passed_ref ) = @_;
    ### <where> - reached before_method
    #### <where> - received input: $passed_ref
    #~ $self->_clear_match_string;
    my  $match_string =
            ($self->_has_match_string) ?
                $self->_match_string :
                '#<--- ' ;
    my  $branch_ref = $passed_ref->{branch_ref}->[-1];
    ### <where> - branch values             : $branch_ref
    ### <where> - current match highlighting: $self->get_match_highlighting
    my $should_print = 0;
    if( $branch_ref->[3] ){
		### <where> - adding secondary levels ...
		### <where> - add tabs ...
		if( !$self->_has_pending_string or
			( 	$self->_has_pending_string and
				($self->_get_pending_string !~ /^\t/) ) ){
			$self->_add_to_pending_string( ("\t" x ($branch_ref->[3])) );
		}
		### <where> - add item ...
		$self->_add_to_pending_string(
			$self->_dispatch_method(
				$item_dispatch,
				$branch_ref->[0],
				$branch_ref->[1],
			)
		);
		### <where> - add inter reference markers ...
		$self->_add_to_pending_string(
			$self->_dispatch_method(
				$inter_ref_dispatch,
				$branch_ref->[0],
			)
		);
		### <where> - add to match string as needed
		$match_string .= $self->_dispatch_method(
			$initial_match_statement_dispatch,
			$branch_ref->[0],
			( exists $passed_ref->{secondary_ref} ),
		);
    }
    ### <where> - match string is: $match_string
    if( is_HashRef( $passed_ref->{primary_ref} ) ){######<----------  ADD New types here
        ### <where> - a HASH ref is next
        $self->_add_to_pending_string( '{' );
        $match_string .=
            ( exists $passed_ref->{secondary_ref} and
            is_HashRef( $passed_ref->{secondary_ref} ) ) ?
                'Ref Type Match' : 'Ref Type Mismatch' ;
        $should_print = 1;
    }elsif( is_ArrayRef( $passed_ref->{primary_ref} ) ){
        ### <where> - an ARRAY ref is next
        $self->_add_to_pending_string( '[' );
        $match_string .=
            ( exists $passed_ref->{secondary_ref} and
            is_ArrayRef( $passed_ref->{secondary_ref} ) ) ?
                'Ref Type Match' : 'Ref Type Mismatch' ;
        $should_print = 1;
    }elsif( $branch_ref and $branch_ref->[0] eq 'SCALAR' ){
        $self->_add_to_pending_string( "'$branch_ref->[1]'," );
		### <where> - checking for scalar match with: $passed_ref
		$match_string .= 
			( exists $passed_ref->{secondary_ref} ) ?
				'Secondary Value Matches' : 
				'Secondary Value Does NOT Match' ;
		### <where> - current match string: $match_string
        $should_print = 1;
    }
    if( ref $passed_ref->{primary_ref} ){
        my $skip_method = 'get_skip_' . (ref $passed_ref->{primary_ref}) . '_ref';
        if( $self->$skip_method ){
            $self->_add_to_pending_string( '# !!! SKIPPED !!!' );
        }
    }
    ### <where> - match string is   : $match_string
    ### <where> - match highlighting: $self->get_match_highlighting
    ### <where> - secondary prexist : $self->_get_had_secondary
    $self->_set_match_string( $match_string ) if $match_string;
    if( $should_print ){
        $self->_print_pending_string;
    }
    ### <where> - current string      : $self->_get_pending_string
    ### <where> - current match string: $self->_match_string
    ### <where> - leaving before_method
	return $passed_ref;
}

sub _print_after_method{
    my ( $self, $passed_ref ) = @_;
    my  $branch_ref = $passed_ref->{branch_ref}->[-1];
    my  $match_string =
            ($self->_has_match_string) ?
                $self->_match_string :
            (!($self->_has_pending_string)) ?
                '#<--- ' : '' ;
    ### <where> - reached after_method
    ##### <where> - received input: $passed_ref
    ### <where> - branchvalues  : $branch_ref
    ### <where> - current match : $match_string
    if( $branch_ref ){
        if( !$self->_has_pending_string ){
            ### <where> - No currently pending string
            if( $branch_ref->[3] and is_Ref( $passed_ref->{primary_ref} ) ){
                $self->_add_to_pending_string( ("\t" x ($branch_ref->[3])) );
            }
        }
    }
    if( is_HashRef( $passed_ref->{primary_ref} ) ){######<----------  ADD New types here
        ### <where> - a HASH ref has just closed ...
        $self->_add_to_pending_string( '}' );
        $match_string .=
            ( exists $passed_ref->{secondary_ref} and
            is_HashRef( $passed_ref->{secondary_ref} ) ) ?
                'Ref Type Match' : 'Ref Type Mismatch' ;
    }elsif( is_ArrayRef( $passed_ref->{primary_ref} ) ){
        ### <where> - an ARRAY ref is just completed
        $self->_add_to_pending_string( ']' );
        $match_string .=
            ( exists $passed_ref->{secondary_ref} and
            is_ArrayRef( $passed_ref->{secondary_ref} ) ) ?
                'Ref Type Match' : 'Ref Type Mismatch' ;
    }
    $self->_print_pending_string( ',' );
    ### <where> - after_method complete
    #### <where> - returning: $passed_ref
    return $passed_ref;
}

sub _add_to_pending_string{
    my ( $self, $string ) = @_;
    ### <where> - reached add to pending string
    ### <where> - adding: $string
    $self->_set_pending_string( 
        (($self->_has_pending_string) ?
            $self->_get_pending_string : '') . 
        ( ( $string ) ? $string : '' )
    );
    return 1;
}

sub _print_pending_string{
    my ( $self, $string ) = @_;
    ### <where> - reached print pending string ...
	#### <where> - match_highlighting called: $self->has_match_highlighting
	#### <where> - match_highlighting on: $self->get_match_highlighting
	#### <where> - secondary_ref exists: $self->_get_had_secondary
	#### <where> - has pending match string: $self->_has_match_string
    if( $self->_has_pending_string ){
        my  $new_string = $self->_get_pending_string;
            $new_string .= $string if $string;
            if(	$self->has_match_highlighting and
                $self->get_match_highlighting and
                $self->_get_had_secondary and
                $self->_has_match_string        ){
                ### <where> - match_highlighting on - adding match string
                $new_string .= $self->_match_string;
            }
            $new_string .= "\n";
        ### <where> - printing string: $new_string
        print  $new_string;
    }
    $self->_clear_pending_string;
    $self->_clear_match_string;
    return 1;
}

#################### Phinish with a Phlourish ###########################################

no Moose::Role;

1;
# The preceding line will help the module return a true value

#################### main pod documentation begin #######################################

__END__

=head1 NAME

Data::Walk::Print - A data printing function

=head1 SYNOPSIS
    
	#!perl
	use Modern::Perl;
	use YAML::Any;
	use Moose::Util qw( with_traits );
	use Data::Walk::Extracted v0.015;
	use Data::Walk::Print v0.009;

	$| = 1;

	#Use YAML to compress writing the data ref
	my  $firstref = Load(
		'---
		Someotherkey:
			value
		Parsing:
			HashRef:
				LOGGER:
					run: INFO
		Helping:
			- Somelevel
			- MyKey:
				MiddleKey:
					LowerKey1: lvalue1
					LowerKey2:
						BottomKey1: 12345
						BottomKey2:
						- bavalue1
						- bavalue2
						- bavalue3'
	);
	my  $secondref = Load(
		'---
		Someotherkey:
			value
		Helping:
			- Somelevel
			- MyKey:
				MiddleKey:
					LowerKey1: lvalue1
					LowerKey2:
						BottomKey2:
						- bavalue1
						- bavalue2
						BottomKey1: 12354'
	);
	my $AT_ST = with_traits( 
			'Data::Walk::Extracted', 
			( 'Data::Walk::Print' ),
		)->new(
			match_highlighting => 1,#This is the default
		);
	$AT_ST->print_data(
		print_ref	=>  $firstref,
		match_ref	=>  $secondref,
		sort_HASH	=> 1,#To force order for demo purposes
	);
    
    #####################################################################################
    #     Output of SYNOPSIS
    # 01:{#<--- Ref Type Match
    # 02:	Helping => [#<--- Secondary Key Match - Ref Type Match
    # 03:		'Somelevel',#<--- Secondary Position Exists - Secondary Value Matches
    # 04:		{#<--- Secondary Position Exists - Ref Type Match
    # 05:			MyKey => {#<--- Secondary Key Match - Ref Type Match
    # 06:				MiddleKey => {#<--- Secondary Key Match - Ref Type Match
    # 07:					LowerKey1 => 'lvalue1',#<--- Secondary Key Match - Secondary Value Matches
    # 08:					LowerKey2 => {#<--- Secondary Key Match - Ref Type Match
    # 09:						BottomKey1 => '12345',#<--- Secondary Key Match - Secondary Value Does NOT Match
    # 10:						BottomKey2 => [#<--- Secondary Key Match - Ref Type Match
    # 11:							'bavalue1',#<--- Secondary Position Exists - Secondary Value Matches
    # 12:							'bavalue2',#<--- Secondary Position Exists - Secondary Value Does NOT Match
    # 13:							'bavalue3',#<--- Secondary Position Does NOT Exist - Secondary Value Does NOT Match
    # 14:						],
    # 15:					},
    # 16:				},
    # 17:			},
    # 18:		},
    # 19:	],
    # 20:	Parsing => {#<--- Secondary Key Mismatch - Ref Type Mismatch
    # 21:		HashRef => {#<--- Secondary Key Mismatch - Ref Type Mismatch
    # 22:			LOGGER => {#<--- Secondary Key Mismatch - Ref Type Mismatch
    # 23:				run => 'INFO',#<--- Secondary Key Mismatch - Secondary Value Does NOT Match
    # 24:			},
    # 25:		},
    # 26:	},
    # 27:	Someotherkey => 'value',#<--- Secondary Key Match - Secondary Value Matches
    # 28:},
    #####################################################################################

 
=head1 DESCRIPTION

This L<Moose::Role|https://metacpan.org/module/Moose::Manual::Roles> is mostly written 
as a demonstration module for L<Data::Walk::Extracted>.  
Both L<Data::Dumper|https://metacpan.org/module/Data::Dumper#Functions> - Dumper and 
L<YAML|https://metacpan.org/module/YAML::Any#SUBROUTINES> - Dump functions are more mature than 
the printing function included here.

=head2 Caveat utilitor

=head3 Supported Node types

=over

=item B<ARRAY>

=item B<HASH>

=item B<SCALAR>

=back

=head3 Supported one shot L</Attributes>

=over

=item match_highlighting

=back

=head2 USE

This is a L<Moose::Role|https://metacpan.org/module/Moose::Manual::Roles> and can be 
used as such.  One way to incorporate this role into L<Data::Walk::Extracted> is the 
method 'with_traits' from L<Moose::Util|https://metacpan.org/module/Moose::Util#EXPORTED-FUNCTIONS>.  
Otherwise see L<Moose::Manual::Roles|https://metacpan.org/module/Moose::Manual::Roles>.

=head2 Attributes

Data passed to ->new when creating an instance using a class.  For modification of 
these attributes see L</Methods>.  The ->new function will either accept fat comma 
lists or a complete hash ref that has the possible appenders as the top keys.  
Additionally L<some attributes|/Supported one shot > that have all the following 
methods; get_$attribute, set_$attribute, has_$attribute, and clear_$attribute,
can be passed to L<print_data|/print_data( $arg_ref|%args )> and will be adjusted for 
just the run of that method call.  These are called 'one shot' attributes.

=head3 match_highlighting

=over

=item B<Definition:> this determines if a comments string is added after each printed 
row that indicates how the 'print_ref' matches the 'match_ref' (or not).

=item B<Default> True (1)

=item B<Range> This is a Boolean data type and generally accepts 1 or 0
    
=back

L<Attributes in Data::Walk::Extracted|http://search.cpan.org/~jandrew/Data-Walk-Extracted/lib/Data/Walk/Extracted.pm#Attributes> 
 - also affect the output.

=head1 Methods

=head2 print_data( $arg_ref|%args )

=over

=item B<Definition:> this is the method used to print a data reference

=item B<Accepts:> either a single data reference or named arguments 
in a fat comma list or hashref

=over

=item B<named variable option> - if data comes in a fat comma list or as a hash ref 
and the keys include a 'print_ref' key then the list is processed as such.

=over

=item B<print_ref> - this is the data reference that should be printed in a perlish way 
- Required

=item B<match_ref> - this is a reference used to compare against the 'print_ref'
- Optional

=item B<[attribute name]> - attribute names are accepted with temporary attribute settings.  
These settings are temporarily set for a single "print_data" call and then the original 
attribute values are restored.  For this to work the the attribute must meet the 
L<necessary criteria|/Supported one shot >.

=back

=item B<single variable option> - if only one data_ref is sent and it fails the test 
for "exists $data_ref->{print_ref}" then the program will attempt to name it as 
print_ref => $data_ref and then process the data as a fat comma list.

=back

=item B<Returns:> 1 (And prints out the data ref with matching assesment comments per 
L</match_highlighting>)

=back

=head2 set_match_highlighting( $bool )

=over

=item B<Definition:> this is a way to change the match_highlighting attribute

=item B<Accepts:> a Boolean value

=item B<Returns:> ''

=back

=head1 GLOBAL VARIABLES

=over

=item B<$ENV{Smart_Comments}>

The module uses L<Smart::Comments> if the '-ENV' option is set.  The 'use' is 
encapsulated in a BEGIN block triggered by the environmental variable to comfort 
non-believers.  Setting the variable $ENV{Smart_Comments} will load and turn 
on smart comment reporting.  There are three levels of 'Smartness' available 
in this module '### #### #####'.

=back

=head1 SUPPORT

=over

=item L<Data-Walk-Extracted/issues|https://github.com/jandrew/Data-Walk-Extracted/issues>

=back

=head1 TODO

=over

=item * Support printing CodeRefs

=item * Support printing Objects / Instances

=item * print the data address if the node is skipped

=back

=head1 AUTHOR

=over

=item Jed Lund

=item jandrew@cpan.org

=back

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 Dependencies

=over

=item L<Data::Walk::Extracted>

=item L<version>

=item L<Moose::Role>

=item L<MooseX::Types::Moose>

=back

=head1 SEE ALSO

=over

=item L<Smart::Comments> - is used if the -ENV option is set

=item L<Data::Walk>

=item L<Data::Walker>

=item L<Data::Dumper> - Dumper

=item L<YAML> - Dump

=back

=cut

#################### main pod documentation end #########################################
